/*****************************************************************************
 *  Rohde&Schwarz LCX LCR Meter instrument driver
 *  Original Release: April 2022
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *    see ChangeLog
 *
 *****************************************************************************/
#include "rslcx.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

/*****************************************************************************
 * Function: rslcx_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rslcx_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rslcx_init(ViRsrc resourceName, ViBoolean IDQuery,
                                 ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rslcx_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rslcx_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rslcx_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean IDQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    ViSession instrSession = 0;
    RsCoreSessionPtr rsSession = NULL;
    ViChar completeOptionString[RS_MAX_MESSAGE_BUF_SIZE];

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    *newInstrSession = 0;

    // TCPIP, USB and GPIB sessions to non-vxi capable
    (void)RsCore_CopyToUserBufferAsciiData(completeOptionString, RS_MAX_MESSAGE_BUF_SIZE, optionString);
    (void)RsCore_AppendToCsvString(completeOptionString, ",", RS_MAX_MESSAGE_BUF_SIZE, "AdditionalFlags=3");

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rslcx", // driver prefix
        completeOptionString, // Init Options string, is applied to the session settings
        rslcx_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rslcx_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "HMC8043", VI_FALSE: "HMC" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        10000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSLCX_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        100000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.4.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 4, 0));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    // --- Here perform settings that are default for this driver, but can be overwritten by the optionsString settings
    rsSession->assureResponseEndWithLF = VI_TRUE;

    // Parse option string and set the desired values
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
        checkErr(RsCore_BuildRepCapTable(instrSession, rslcx_RsCoreRepCapTable, rslcx_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "HMC8043", false : "HMC"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSLCX_SIMULATION_ID_QUERY, NULL));

    // Not all instruments support *OPT?, therefore skip the *OPT? query

    // Skipping option checking, until instrument's FW fix for *OPT? is available
    // Current instrument's response is K0(K106+K107+K108+K201+K210)
    // Expected format is K0,K106,K107,K108,K201,K210 or K106,K107,K108,K201,K210
    // Once fixed, the following disabled optionChecking can be deleted
    rsSession->optionChecking = VI_FALSE;

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rslcx_reset(instrSession));
    }
    else
    {
        checkErr(rslcx_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSLCX_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && rsSession->instrStatusStbQueryFirst == VI_TRUE && rsSession->autoSystErrQuery == VI_FALSE)
        {
            rsSession->autoSystErrQuery = VI_TRUE;
            (void)rslcx_CheckStatus(instrSession);
            rsSession->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN Configures the frequency of the test signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Sets the frequency of the test signal.
ViStatus _VI_FUNC rslcx_ConfigureFrequency(ViSession instrSession,
                                           ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the test signal voltage in RMS (root mean square).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR voltageLevel/Sets the test signal voltage in RMS (root mean square).
ViStatus _VI_FUNC rslcx_ConfigureVoltageLevel(ViSession instrSession,
                                              ViReal64  voltageLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_VOLTAGE_LEVEL, voltageLevel),
            2, "Voltage Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the test signal current in RMS (root mean square).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR currentLevel/Sets the test signal current in RMS (root mean square).
ViStatus _VI_FUNC rslcx_ConfigureCurrentLevel(ViSession instrSession,
                                              ViReal64  currentLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_CURRENT_LEVEL, currentLevel),
            2, "Current Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the main instrument settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autorangeEnabled/Activates automatic impedance range selection.
/// HIPAR manualRangeValue/Sets the impedance range value.
ViStatus _VI_FUNC rslcx_ConfigureRange(ViSession instrSession,
                                       ViBoolean autorangeEnabled,
                                       ViReal64  manualRangeValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_AUTORANGE_ENABLED, autorangeEnabled),
            2, "Autorange Enabled");
    if (!autorangeEnabled)
    {
        viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_MANUAL_RANGE_VALUE, manualRangeValue),
                3, "Manual Range Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures loggine file name and mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Sets the file name and path for the storing the data recorded during data logging. The query returns the file name and path. You can query the information also when data logging is running.
/// HIPAR mode/Sets the logging mode.
ViStatus _VI_FUNC rslcx_ConfigureLogging(ViSession instrSession,
                                         ViString  fileName,
                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LOGGING_FILE_NAME, fileName),
            2, "File Name");

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_LOGGING_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time period for data logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interval/Sets the time period for data logging.
ViStatus _VI_FUNC rslcx_ConfigureLoggingInterval(ViSession instrSession,
                                                 ViReal64  interval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_LOGGING_INTERVAL, interval),
            2, "Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of measurement readings for data logging Mode > Count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Sets the number of measurement readings for data logging Mode > Count.
ViStatus _VI_FUNC rslcx_ConfigureLoggingCount(ViSession instrSession,
                                              ViInt32   count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_LOGGING_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time period for data logging Mode > Duration or Mode > Span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR duration/Sets the time period for data logging Mode > Duration or Mode > Span.
ViStatus _VI_FUNC rslcx_ConfigureLoggingDuration(ViSession instrSession,
                                                 ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_LOGGING_DURATION, duration),
            2, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time for starting data logging in Mode > Span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Year.
/// HIPAR month/Month.
/// HIPAR day/Day.
/// HIPAR hour/Hour.
/// HIPAR minute/Minute.
/// HIPAR second/Second.
ViStatus _VI_FUNC rslcx_ConfigureLoggingStartTime(ViSession instrSession,
                                                  ViInt32   year,
                                                  ViInt32   month,
                                                  ViInt32   day,
                                                  ViInt32   hour,
                                                  ViInt32   minute,
                                                  ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 2010, 2037),
            2, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            5, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            6, "Minute");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, second, 0, 59),
            7, "Second");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld,%ld,%ld,%ld,%ld",
            year, month, day, hour, minute, second);
    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LOGGING_START_TIME, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the data logging function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activates/Deactivates the data logging function.
ViStatus _VI_FUNC rslcx_ConfigureLoggingEnabled(ViSession instrSession,
                                                ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_LOGGING_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the DI measurement sweep parameters. Selects the sweep
/// HIFN  parameter and the sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepParameter/Selects the measurement parameter that varies in defined steps within the sweep range.
/// HIPAR minimum/Sets the first value of the range used for the selected sweep parameter.
/// HIPAR maximum/Sets the last value of the range used for the selected sweep parameter.
ViStatus _VI_FUNC rslcx_ConfigureDIMeasurementSweep(ViSession instrSession,
                                                    ViInt32   sweepParameter,
                                                    ViReal64  minimum,
                                                    ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_SWEEP_PARAMETER, sweepParameter),
            2, "Sweep Parameter");

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_MAXIMUM, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the mode to determine the measurement steps within the sweep
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR intervalType/Selects the mode to determine the measurement steps within the sweep
/// HIPAR intervalType/range.
ViStatus _VI_FUNC rslcx_ConfigureDIMeasurementIntervalType(ViSession instrSession,
                                                           ViInt32   intervalType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_INTERVAL_TYPE, intervalType),
            2, "Interval Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the interval between each sweep step within the measurement range
/// HIFN  Min/Max).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stepSize/Sets the interval between each sweep step within the measurement range Min/Max.
ViStatus _VI_FUNC rslcx_ConfigureDIMeasurementStepSize(ViSession instrSession,
                                                       ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_STEP_SIZE, stepSize),
            2, "step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of measurement points within the measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfPoints/Sets the number of measurement points within the measurement range.
ViStatus _VI_FUNC rslcx_ConfigureDIMeasurementNumberOfPoints(ViSession instrSession,
                                                             ViInt32   numberOfPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_NUMBER_OF_POINTS, numberOfPoints),
            2, "Number Of Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a dynamic impedance measurement with selected parameters
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_DIMeasurementExecute(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Aborts a running dynamic impedance measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_DIMeasurementAbort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_DI_MEASUREMENT_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the binning measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR binningEnabled/Activates the binning measurement.
ViStatus _VI_FUNC rslcx_ConfigureBinningEnabled(ViSession instrSession,
                                                ViBoolean binningEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_BINNING_ENABLED, binningEnabled),
            2, "Binning Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Uploads the binning configuration file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/Uploads the binning configuration file.
ViStatus _VI_FUNC rslcx_LoadBinningFromFile(ViSession instrSession,
                                            ViString  filePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_BINNING_LOAD_FROM_FILE, filePath),
            2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of samples counted in the bins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR binningSamples/Queries the number of samples counted in the bins.
ViStatus _VI_FUNC rslcx_QueryBinningSamples(ViSession instrSession,
                                            ViInt32*  binningSamples)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_BINNING_SAMPLES, binningSamples),
            2, "Binning Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total number of samples measured since reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR binningSamplesSinceReset/Queries the total number of samples measured since reset.
ViStatus _VI_FUNC rslcx_QueryBinningSamplesSinceReset(ViSession instrSession,
                                                      ViInt32*  binningSamplesSinceReset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_BINNING_SAMPLES_SINCE_RESET, binningSamplesSinceReset),
            3, "Binning Samples Since Reset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reset binning statistic
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_ResetBinningCounter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_BINNING_COUNTER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set/Query the measurement mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Set/Query the measurement mode
ViStatus _VI_FUNC rslcx_ConfigureMeasurementMode(ViSession instrSession,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the measurement function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the measurement function.
ViStatus _VI_FUNC rslcx_ConfigureMeasurementType(ViSession instrSession,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures measurement parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementSpeed/Selects the speed of the measurement repetitions per second.
/// HIPAR cableLength/Sets the length of the leads to the connected test fixture i.e. the
/// HIPAR cableLength/DUT.
/// HIPAR voltageBias/Activates the externally supplied bias voltage.
/// HIPAR sourceImpedance/Enable/disable LowZ mode. With LowZ mode enabled, R-source is reduced
/// HIPAR sourceImpedance/to 10 Ohm. (Default 100 Ohm)
/// HIPAR triggerDelay/Set/Query the trigger delay
ViStatus _VI_FUNC rslcx_ConfigureMeasurement(ViSession instrSession,
                                             ViInt32   measurementSpeed,
                                             ViInt32   cableLength,
                                             ViInt32   voltageBias,
                                             ViInt32   sourceImpedance,
                                             ViReal64  triggerDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_SPEED, measurementSpeed),
            2, "Measurement Speed");

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CABLE_LENGTH, cableLength),
            3, "Cable Length");

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL,
        RSLCX_ATTR_BIAS_EXTERNAL_ENABLED, voltageBias == RSLCX_VAL_EXTERNAL), 4, "Voltage Bias");

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL,
        RSLCX_ATTR_MEASUREMENT_LOW_Z_MODE_ENABLED, sourceImpedance == RSLCX_VAL_IMPEDANCE_LOW), 5, "Source Impedance");

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_TRIGGER_DELAY, triggerDelay),
            6, "Trigger Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the impedance parameter for the measurement corresponding to
/// HIFN  the measurement type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR impedanceType/Selects the impedance parameter for the measurement corresponding to
/// HIPAR impedanceType/the measurement type.
ViStatus _VI_FUNC rslcx_ConfigureMeasurementImpedanceType(ViSession instrSession,
                                                          ViInt32   impedanceType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_IMPEDANCE_TYPE, impedanceType),
            2, "Impedance Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new measurement.
/// HIFN     In manual trigger mode, the command triggers a single measurement
/// HIFN  cycle. When completed, the R&S LCX waits for the next trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_MeasurementInit(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_INIT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the internal voltage or current bias.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR biasEnabled/Activates the internal voltage or current bias.
ViStatus _VI_FUNC rslcx_ConfigureBiasEnabled(ViSession instrSession,
                                             ViBoolean biasEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_BIAS_ENABLED, biasEnabled),
            2, "Bias Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the internal bias current value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR currentBias/Sets the internal bias current value.
ViStatus _VI_FUNC rslcx_ConfigureCurrentBias(ViSession instrSession,
                                             ViReal64  currentBias)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_CURRENT_BIAS, currentBias),
            2, "Current Bias");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the internal DC bias voltage value. Maximum is dependent on test
/// HIFN  signal frequency. (Max 2V when freq > 2 MHz)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR voltageBias/Sets the internal DC bias voltage value. Maximum is dependent on test
/// HIPAR voltageBias/signal frequency. (Max 2V when freq > 2 MHz)
ViStatus _VI_FUNC rslcx_ConfigureVoltageBias(ViSession instrSession,
                                             ViReal64  voltageBias)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_VOLTAGE_BIAS, voltageBias),
            3, "Voltage Bias");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the value of the externally applied voltage.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR biasMeasureExternal/Queries the value of the externally applied voltage.
ViStatus _VI_FUNC rslcx_QueryBiasMeasureExternal(ViSession instrSession,
                                                 ViReal64* biasMeasureExternal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_BIAS_MEASURE_EXTERNAL, biasMeasureExternal),
            2, "Bias Measure External");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/Deactivates the open/short/load correction function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR openEnabled/Activates/Deactivates the open correction function.
/// HIPAR shortEnabled/Activates/Deactivates the short correction function.
/// HIPAR loadEnabled/Activates/Deactivates the load correction function.
ViStatus _VI_FUNC rslcx_ConfigureMeasurementCorrection(ViSession instrSession,
                                                       ViBoolean openEnabled,
                                                       ViBoolean shortEnabled,
                                                       ViBoolean loadEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_OPEN_ENABLED, openEnabled),
            2, "Open Enabled");

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_SHORT_ENABLED, shortEnabled),
            3, "Short Enabled");

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_LOAD_ENABLED, loadEnabled),
            4, "Load Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes an open correction on all frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_StartMeasurementOpenCorrectionFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_OPEN_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a short correction on all frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_StartMeasurementShortCorrectionFull(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_SHORT_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current state and mode of open correction, if enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIFN      rslcx_InitWithOptions function. The handle identifies a particular
/// HIFN      instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR result/Queries the current state and mode of open correction, if enabled.
ViStatus _VI_FUNC rslcx_QueryMeasurementCorrectionOpenMode(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    result[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_OPEN_MODE,
            bufferSize, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current state and mode of short correction, if enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIFN      rslcx_InitWithOptions function. The handle identifies a particular
/// HIFN      instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR result/Queries the current state and mode of short correction, if enabled.
ViStatus _VI_FUNC rslcx_QueryMeasurementCorrectionShortMode(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    result[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_SHORT_MODE,
            bufferSize, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current state and mode of load correction, if enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIFN      rslcx_InitWithOptions function. The handle identifies a particular
/// HIFN      instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR result/Queries the current state and mode of load correction, if enabled.
ViStatus _VI_FUNC rslcx_QueryMeasurementCorrectionLoadMode(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    result[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_MEASUREMENT_CORRECTION_LOAD_MODE,
            bufferSize, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates and configures the specified working point for the correction
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR workingPoint/Dedicated working point.
/// HIPAR enabled/Activates the specified working point for the correction function.
/// HIPAR frequency/Assign the working point number and the frequency.
ViStatus _VI_FUNC rslcx_ConfigureMeasurementCorrectionSpot(ViSession instrSession,
                                                           ViInt32   workingPoint,
                                                           ViBoolean enabled,
                                                           ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, workingPoint, 0), 2, "Working Point");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:STAT %s", workingPoint, enabled?"ON":"OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:FREQ %.12lG", workingPoint, frequency);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a working point for load correction.
/// HIFN     Assign the working point number and the primary and secondary
/// HIFN  reference values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR workingPoint/Dedicated working point.
/// HIPAR primary/Sets the primary standard value as reference, e.g. the value of a
/// HIPAR primary/calibration resistor.
/// HIPAR secondary/Sets the secondary standard value.
ViStatus _VI_FUNC rslcx_ConfigureMeasurementCorrectionSpotLoad(
                            ViSession instrSession,
                            ViInt32   workingPoint,
                            ViReal64  primary,
                            ViReal64  secondary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, workingPoint, 0), 2, "Working Point");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:LOAD:STAN %.12lG,%.12lG", workingPoint, primary, secondary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a load correction at a dedicated working point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR workingPoint/Dedicated working point.
ViStatus _VI_FUNC rslcx_StartMeasurementSpotLoadCorrection(ViSession instrSession,
                                                           ViInt32   workingPoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, workingPoint, 0), 2, "Working Point");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:LOAD", workingPoint);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a short correction at a dedicated working point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR workingPoint/Dedicated working point.
ViStatus _VI_FUNC rslcx_StartMeasurementSpotShortCorrection(ViSession instrSession,
                                                            ViInt32   workingPoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, workingPoint, 0), 2, "Working Point");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:SHOR", workingPoint);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a open correction at a dedicated working point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR workingPoint/Dedicated working point.
ViStatus _VI_FUNC rslcx_StartMeasurementSpotOpenCorrection(ViSession instrSession,
                                                           ViInt32   workingPoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, workingPoint, 0), 2, "Working Point");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CORR:SPOT%ld:OPEN", workingPoint);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the next voltage measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nextVoltage/Query the next voltage measurement
ViStatus _VI_FUNC rslcx_QueryNextVoltage(ViSession instrSession,
                                         ViReal64* nextVoltage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_NEXT_VOLTAGE, nextVoltage),
            2, "Next Voltage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the next voltage measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nextCurrent/Query the next voltage measurement
ViStatus _VI_FUNC rslcx_QueryNextCurrent(ViSession instrSession,
                                         ViReal64* nextCurrent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_NEXT_CURRENT, nextCurrent),
            2, "Next Current");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent valid values for measurement pair 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR z/Returns value of Z.
/// HIPAR thetad/Returns value of Theta d.
ViStatus _VI_FUNC rslcx_Fetch(ViSession instrSession,
                              ViReal64* z,
                              ViReal64* thetad)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViReal64(instrSession, "FETC?", z, thetad, NULL, NULL));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the most recent valid values of the measured impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR z/Returns value of Z.
/// HIPAR thetad/Returns value of Theta d.
ViStatus _VI_FUNC rslcx_FetchImpedance(ViSession instrSession,
                                       ViReal64* z,
                                       ViReal64* thetad)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViReal64(instrSession, "FETC:IMP?", z, thetad, NULL, NULL));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the measurement results for  measurement pair 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR z/Returns value of Z.
/// HIPAR thetad/Returns value of Theta d.
ViStatus _VI_FUNC rslcx_Read(ViSession instrSession,
                             ViReal64* z,
                             ViReal64* thetad)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViReal64(instrSession, "READ?", z, thetad, NULL, NULL));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the impedance measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR z/Returns value of Z.
/// HIPAR thetad/Returns value of Theta d.
ViStatus _VI_FUNC rslcx_ReadImpedance(ViSession instrSession,
                                      ViReal64* z,
                                      ViReal64* thetad)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViReal64(instrSession, "READ:IMP?", z, thetad, NULL, NULL));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the accuracy of the last measurement. Accuracy is given in
/// HIFN  percent for |Z| and in degree for Thetad.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR z/Measurement accuracy of Z.
/// HIPAR thetad/Measurement accuracy of Theta d.
ViStatus _VI_FUNC rslcx_QueryMeasurementAccuracy(ViSession instrSession,
                                                 ViReal64* z,
                                                 ViReal64* thetad)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryTupleViReal64(instrSession, "MEAS:ACC?", z, thetad, NULL, NULL));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure front key brightness.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR brightness/Configure front key brightness.
ViStatus _VI_FUNC rslcx_ConfigureFrontKeyBrightness(ViSession instrSession,
                                                    ViReal64  brightness)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_FRONT_KEY_BRIGHTNESS, brightness),
            2, "Brightness");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the USB communication class.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR usbClass/Selects the USB communication class.
ViStatus _VI_FUNC rslcx_ConfigureUSBClass(ViSession instrSession,
                                          ViInt32   usbClass)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_USB_CLASS, usbClass),
            2, "USB Class");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is
/// HIFN  relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rslcx_SystemVersion(ViSession instrSession,
                                      ViInt32   length,
                                      ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, "", RSLCX_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            3, "System Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns device foot print info
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Device Foot Print string.
/// HIPAR systemDeviceFootPrint/Returns device foot print info
ViStatus _VI_FUNC rslcx_SystemDeviceFootPrint(ViSession instrSession,
                                              ViInt32   length,
                                              ViChar    systemDeviceFootPrint[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, "", RSLCX_ATTR_SYSTEM_DEVICE_FOOT_PRINT,
            length, systemDeviceFootPrint),
            3, "System Device Foot Print");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query the device hardware version
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR systemHWVersion/Query the device hardware version
ViStatus _VI_FUNC rslcx_SystemHWVersion(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    systemHWVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, "", RSLCX_ATTR_SYSTEM_HW_VERSION,
            bufferSize, systemHWVersion),
            3, "System HW Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The front panel control is locked; with the SystemLocal method the
/// HIFN  front panel control will be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_SystemRWLock(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_RW_LOCK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Local operating mode (front panel control).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_SystemLocal(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_LOCAL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Remote control (front panel control locked).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_SystemRemote(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_REMOTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restart the device
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_SystemRestart(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_RESTART, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the date of the internal calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/Year, to be entered as a four-digit number (including the century and
/// HIPAR year/millennium information)
/// HIPAR month/Month, 1 (January) to 12 (December)
/// HIPAR day/Day, 1 to the maximum number of days in the specified month
ViStatus _VI_FUNC rslcx_SetSystemDate(ViSession instrSession,
                                      ViInt32   year,
                                      ViInt32   month,
                                      ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 2010, 2037),
            2, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld,%ld", year, month, day);
    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_DATE, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the date of the internal calendar. String is in the format
/// HIFN  '<Year>,<Month>,<Day>'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR systemDate/Returns the date of the internal calendar. String is in the format
/// HIPAR systemDate/'<Year>,<Month>,<Day>'.
ViStatus _VI_FUNC rslcx_QuerySystemDate(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    systemDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_GetAttributeViString(instrSession, NULL,
        RSLCX_ATTR_SYSTEM_DATE, bufferSize, systemDate));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the internal time
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hour/Hour
/// HIPAR minute/Minute
/// HIPAR second/Second
ViStatus _VI_FUNC rslcx_SetSystemTime(ViSession instrSession,
                                      ViInt32   hour,
                                      ViInt32   minute,
                                      ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            2, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            3, "Minute");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, second, 0, 59),
            4, "Second");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld,%ld", hour, minute, second);
    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_TIME, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the UTC (Universal Time Coordinated) of the internal clock. To
/// HIFN  define the current local time, use the time zone setting of the
/// HIFN  operating system (Setup > Time, date). Return string is of the format
/// HIFN  '<Hour>,<Minute>,<Second>'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR systemTime/Returns the UTC (Universal Time Coordinated) of the internal clock. To
/// HIPAR systemTime/define the current local time, use the time zone setting of the
/// HIPAR systemTime/operating system (Setup > Time, date). Return string is of the format
/// HIPAR systemTime/'<Hour>,<Minute>,<Second>'.
ViStatus _VI_FUNC rslcx_QuerySystemTime(ViSession instrSession,
                                        ViInt32   bufferSize,
                                        ViChar    systemTime[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_TIME,
            bufferSize, systemTime),
            3, "System Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query system uptime
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR systemUptime/Query system uptime
ViStatus _VI_FUNC rslcx_QuerySystemUptime(ViSession instrSession,
                                          ViInt32   bufferSize,
                                          ViChar    systemUptime[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_SYSTEM_UPTIME,
            bufferSize, systemUptime),
            3, "System Uptime");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the VNC interface for remote access.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activates the VNC interface for remote access.
/// HIPAR port/Sets the VNC port address.
ViStatus _VI_FUNC rslcx_ConfigureVNC(ViSession instrSession,
                                     ViBoolean enabled,
                                     ViInt32   port)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_VNC_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_FALSE)
    {
        viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_VNC_PORT, port),
                3, "Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure device display brightness.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR brightness/Configure device display brightness.
ViStatus _VI_FUNC rslcx_ConfigureDisplayBrightness(ViSession instrSession,
                                                   ViReal64  brightness)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViReal64(instrSession, NULL, RSLCX_ATTR_DISPLAY_BRIGHTNESS, brightness),
            2, "Brightness");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Show custom text in a message box on front GUI.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userMessageText/Show custom text in a message box on front GUI.
ViStatus _VI_FUNC rslcx_ConfigureDisplayUserMessage(ViSession instrSession,
                                                    ViString  userMessageText)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_DISPLAY_USER_MESSAGE, userMessageText),
            2, "User Message Text");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Closes a user defined text message on the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_DisplayUserMessageClose(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_DISPLAY_USER_MESSAGE_CLOSE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures acoustic signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR warningEnabled/Activates the R&S LCX to create an acoustic signal on errors and
/// HIPAR warningEnabled/warnings.
/// HIPAR operationCompleteEnabled/Activates the R&S LCX to create an acoustic signal on a completed
/// HIPAR operationCompleteEnabled/operation.
ViStatus _VI_FUNC rslcx_ConfigureBeeper(ViSession instrSession,
                                        ViBoolean warningEnabled,
                                        ViBoolean operationCompleteEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_BEEPER_WARNING_ENABLED, warningEnabled),
            2, "Warning Enabled");

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_BEEPER_OPERATION_COMPLETE_ENABLED, operationCompleteEnabled),
            3, "Operation Complete Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates that the R&S LCX issues a beep on error or warning
/// HIFN  immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_WarningBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_BEEPER_WARNING, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates that the R&S LCX issues a beep after operation complete
/// HIFN  immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_OperationCompleteBeep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_BEEPER_OPERATION_COMPLETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Gets screenshot data in binary format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR destination/Defines destination file to which the data transfer from instrument to
/// HIPAR destination/control computer applies.
ViStatus _VI_FUNC rslcx_ImageSnapshot(ViSession instrSession,
                                      ViString  destination)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, destination), 2, "Destination");

    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, "HCOP:DATA?", destination, RS_VAL_TRUNCATE));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure the file format used for screenshots
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageFormat/Configure the file format used for screenshots
ViStatus _VI_FUNC rslcx_ConfigureImageFormat(ViSession instrSession,
                                             ViInt32   imageFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_IMAGE_FORMAT, imageFormat),
            3, "Image Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the width and height of the screenshot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR imageWidth/Query the screen width
/// HIPAR imageHeight/Query the screen height
ViStatus _VI_FUNC rslcx_QueryImageSize(ViSession instrSession,
                                       ViInt32*  imageWidth,
                                       ViInt32*  imageHeight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_IMAGE_WIDTH, imageWidth),
            2, "Image Width");

    viCheckParm(rslcx_GetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_IMAGE_HEIGHT, imageHeight),
            3, "Image Height");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  En/Disable DHCP for LAN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dhcpEnabled/En/Disable DHCP for LAN
ViStatus _VI_FUNC rslcx_ConfigureLANDHCPEnabled(ViSession instrSession,
                                                ViBoolean dhcpEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_LAN_DHCP_ENABLED, dhcpEnabled),
            2, "DHCP Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the IP address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Sets the IP address.
ViStatus _VI_FUNC rslcx_ConfigureLANIPAddress(ViSession instrSession,
                                              ViString  ipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_IP_ADDRESS, ipAddress),
            2, "IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set/Query subnet mask
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subnetMask/Set/Query subnet mask
ViStatus _VI_FUNC rslcx_ConfigureLANSubnetMask(ViSession instrSession,
                                               ViString  subnetMask)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_SUBNET_MASK, subnetMask),
            2, "Subnet Mask");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set/Query default gateway
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR defaultGateway/Set/Query default gateway
ViStatus _VI_FUNC rslcx_ConfigureLANDefaultGateway(ViSession instrSession,
                                                   ViString  defaultGateway)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_DEFAULT_GATEWAY, defaultGateway),
            2, "Default Gateway");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Set/Query hostname
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hostname/Set/Query hostname
ViStatus _VI_FUNC rslcx_ConfigureLANHostname(ViSession instrSession,
                                             ViString  hostname)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_HOSTNAME, hostname),
            2, "Hostname");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Query MAC address
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR macAddress/Query MAC address
ViStatus _VI_FUNC rslcx_QueryLANMACAddress(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    macAddress[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_MAC_ADDRESS,
            bufferSize, macAddress),
            3, "MAC Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates whether changed LAN parameters have been applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parametersEdited/Indicates whether changed LAN parameters have been applied.
ViStatus _VI_FUNC rslcx_QueryLANParametersEdited(ViSession  instrSession,
                                                 ViBoolean* parametersEdited)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViBoolean(instrSession, NULL, RSLCX_ATTR_LAN_PARAMETERS_EDITED, parametersEdited),
            2, "Parameters Edited");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Apply LAN settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_LANApplyChanges(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_APPLY_CHANGES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Discard LAN settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_LANDiscardChanges(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_DISCARD_CHANGES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reset LAN settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_LANResetParameters(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_LAN_RESET_PARAMETERS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR save/Saves settings.
ViStatus _VI_FUNC rslcx_SaveInstrumentSettings(ViSession instrSession,
                                               ViInt32   save)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_SAVE, save),
            2, "Save");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Recalls settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recall/Recalls settings.
ViStatus _VI_FUNC rslcx_RecallInstrumentSettings(ViSession instrSession,
                                                 ViInt32   recall)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, NULL, RSLCX_ATTR_RECALL, recall),
            2, "Recall");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current instrument settings in a file with defined filename.
/// HIFN     To recall an instrument configuration, use command Recall.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Saves the current instrument settings in a file with defined filename.
ViStatus _VI_FUNC rslcx_SaveInstrumentSettingsToFile(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_SAVE_TO_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sourceInstrumentFile/This control selects file for the data transfer from instrument to
/// HIPAR sourceInstrumentFile/control computer.
/// HIPAR destinationPCFile/This control defines destination file to which the data transfer from
/// HIPAR destinationPCFile/instrument to control computer applies.
ViStatus _VI_FUNC rslcx_ReadToFileFromInstrument(ViSession instrSession,
                                                 ViString  sourceInstrumentFile,
                                                 ViString  destinationPCFile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:DATA? \"%s\"", sourceInstrumentFile);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destinationPCFile, RS_VAL_TRUNCATE));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Delete the specified file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/Delete the specified file
ViStatus _VI_FUNC rslcx_FileDelete(ViSession instrSession,
                                   ViString  filePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViString(instrSession, NULL, RSLCX_ATTR_FILE_DELETE, filePath),
            2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  List all available files
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR listOfFile/List all available files
ViStatus _VI_FUNC rslcx_QueryListOfFiles(ViSession instrSession,
                                         ViInt32   bufferSize,
                                         ViChar    listOfFiles[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViString(instrSession, NULL, RSLCX_ATTR_LIST_OF_FILES,
            bufferSize, listOfFiles),
            3, "List Of File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of measurement readings saved in a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/Path to CSV file with measurements.
/// HIPAR numberOfMeasurements/Read the number of measurements from the specified file
ViStatus _VI_FUNC rslcx_QueryNumberOfMeasurementsInFile(ViSession instrSession,
                                                        ViString  filePath,
                                                        ViInt32*  numberOfMeasurements)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "DATA:POIN? \"%s\"", filePath);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfMeasurements));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rslcx_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
/* ViInt32 */
ViStatus _VI_FUNC rslcx_SetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeID,
                                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rslcx_SetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeID,
                                             ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rslcx_SetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeID,
                                             ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rslcx_SetAttributeViBoolean(ViSession instrSession,
                                              ViString  repeatedCapabilityName,
                                              ViAttr    attributeID,
                                              ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rslcx_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rslcx_GetAttributeViInt32(ViSession instrSession,
                                            ViString  repeatedCapabilityName,
                                            ViAttr    attributeID,
                                            ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rslcx_GetAttributeViReal64(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeID,
                                             ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rslcx_GetAttributeViString(ViSession instrSession,
                                             ViString  repeatedCapabilityName,
                                             ViAttr    attributeID,
                                             ViInt32   bufferSize,
                                             ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rslcx_GetAttributeViBoolean(ViSession  instrSession,
                                              ViString   repeatedCapabilityName,
                                              ViAttr     attributeID,
                                              ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rslcx_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                          ViAttr    attributeID,
                                                          ViInt32   bufferSize,
                                                          ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rslcx_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rslcx_GetAttributeRepCapName(ViSession instrSession,
                                               ViAttr    attributeId,
                                               ViString  repCapNameId,
                                               ViInt32   index,
                                               ViInt32   bufferSize,
                                               ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument. The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rslcx_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rslcx_DefaultInstrSetup(instrSession));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function runs the instrument's self test routine and returns the
/// HIFN  test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR testResult/This control contains the value returned from the instrument self
/// HIPAR testResult/test.  Zero means success.  For any other code, see the device's
/// HIPAR testResult/operator's manual.
/// HIPAR testMessage/Self-test message
ViStatus _VI_FUNC rslcx_self_test(ViSession instrSession,
                                  ViInt16   *testResult,
                                  ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the revision numbers of the instrument driver
/// HIFN  and instrument firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentDriverRevision/Returns the instrument driver software revision numbers in the form of
/// HIPAR instrumentDriverRevision/a string.
/// HIPAR firmwareRevision/Returns the instrument firmware revision numbers in the form of a
/// HIPAR firmwareRevision/string.
ViStatus _VI_FUNC rslcx_revision_query(ViSession instrSession,
                                       ViChar    instrumentDriverRevision[],
                                       ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads an error code and a message from the instrument's
/// HIFN  error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error
/// HIPAR errorMessage/message queue.
ViStatus _VI_FUNC rslcx_error_query(ViSession instrSession,
                                    ViInt32*  errorCode,
                                    ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rslcx_error_message(ViSession instrSession,
                                      ViStatus  errorCode,
                                      ViChar    errorMessage[])
{
    return rslcx_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN  This method resets the edge detectors and ENABle parts of all
/// HIFN  registers to a defined value. All PTRansition parts are set to FFFFh,
/// HIFN  i.e. all transitions from 0 to 1 are detected. All NTRansition parts
/// HIFN  are set to 0, i.e. a transition from 1 to 0 in a CONDition bit is not
/// HIFN  detected. The ENABle part of the STATus:OPERation and
/// HIFN  STATus:QUEStionable registers are set to 0, i.e. all events in these
/// HIFN  registers are not passed on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_StatusPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_SetAttributeViString(instrSession, "", RSLCX_ATTR_STATUS_PRESET, NULL));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method sets the Enable, NTransition, and PTransition bits of
/// HIFN  status questionable and operating registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/Specifies the operation to perform.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR ptransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR ptransition/register.
/// HIPAR ntransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR ntransition/register.
ViStatus _VI_FUNC rslcx_SetStatusRegister(ViSession instrSession,
                                          ViInt32   registerGroup,
                                          ViInt32   enable,
                                          ViInt32   ptransition,
                                          ViInt32   ntransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViPChar  reg_str;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsLcx_rngStatusRegisterOperation, registerGroup, &reg_str));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:ENAB %ld", reg_str, enable);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:PTR %ld", reg_str, ptransition);
    checkErr(RsCore_Write(instrSession, cmd));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:NTR %ld", reg_str, ntransition);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This method contains the commands for the querying status reporting
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR registerGroup/Specifies the operation to perform.
/// HIPAR registerType/This parameter selects the registers to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rslcx_GetStatusRegister(ViSession instrSession,
                                          ViInt32   registerGroup,
                                          ViInt32   registerType,
                                          ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViPChar  reg_str;
    ViPChar  reg_type_str;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsLcx_rngStatusRegisterOperation, registerGroup, &reg_type_str));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsLcx_rngStatusRegisterQuery, registerType, &reg_str));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:%s:%s?", reg_type_str, reg_str);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rslcx_IDQueryResponse(ViSession instrSession,
                                        ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_QueryViString(instrSession, "*IDN?", 1024, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rslcx_QueryOPC(ViSession instrSession,
                                 ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rslcx_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rslcx_ConfigureErrorChecking(ViSession instrSession,
                                               ViBoolean optionChecking,
                                               ViBoolean rangeChecking,
                                               ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rslcx_SetVISATimeout(ViSession instrSession,
                                       ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rslcx_GetVISATimeout(ViSession instrSession,
                                       ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rslcx_SetOPCTimeout(ViSession instrSession,
                                      ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgprf_init or
/// HIPAR instrSession/rscmwgprf_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rslcx_GetOPCTimeout(ViSession instrSession,
                                      ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rslcx_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rslcx_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rslcx_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                    ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rslcx_ConfigureMultiThreadLocking(ViSession instrSession,
                                                    ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN  Retrieves detailed description of the last error that occurred in the
/// HIFN  entered driver session. The latest error overwrites the description. If
/// HIFN  you enter NULL as the instrSession parameter, the driver returns
/// HIFN  the last error that occured in the current execution thread.
/// HIFN     The function also clears the error - calls the rslcx_ClearError() at
/// HIFN  the end.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR code/Return value - returns the last status code. If the returned code is
/// HIPAR code/0, no error occurred since the last call of the rslcx_GetError() or
/// HIPAR code/rslcx_ClearError()
/// HIPAR buffersize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR buffersize/Description parameter. If the error description is longer, it is
/// HIPAR buffersize/truncated and properly 0-terminated.
/// HIPAR description/Returns the error description for the session or execution thread. If
/// HIPAR description/there is no description, the function returns an empty string.
ViStatus _VI_FUNC rslcx_GetError(ViSession instrSession,
                                 ViStatus* code,
                                 ViInt32   bufferSize,
                                 ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));  

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, code), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rslcx_utility_error_message, code, bufferSize, description)
    );

Error:
    // Unlock Session
    // IGNORE: ERROR!!! Missing Lock
    return error;
}

/// HIFN  Clears the information about the last error that occured for the
/// HIFN  entered session. If you enter NULL as the instrSession, the
/// HIFN  function clears the error for the current execution thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN  This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rslcx_WriteInstrData(ViSession instrSession,
                                       ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rslcx_QueryViBoolean(ViSession  instrSession,
                                       ViString   command,
                                       ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value),
        3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus _VI_FUNC rslcx_QueryViInt32(ViSession instrSession,
                                     ViString  command,
                                     ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus _VI_FUNC rslcx_QueryViReal64(ViSession instrSession,
                                      ViString  command,
                                      ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rslcx_QueryViString(ViSession instrSession,
                                      ViString  command,
                                      ViInt32   bufferSize,
                                      ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Cmd (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, command, bufferSize, value, NULL));

    checkErr(rslcx_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function performs the following operations:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rslcx_init or
/// HIPAR instrSession/rslcx_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rslcx_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
